/********************************** (C) COPYRIGHT *******************************
* File Name          : heartrateservice.h
* Author             : WCH
* Version            : V1.0
* Date               : 2018/12/11
* Description        : 
            
*******************************************************************************/

#ifndef SCALESERVICE_H
#define SCALESERVICE_H

#ifdef __cplusplus
extern "C"
{
#endif

/*********************************************************************
 * INCLUDES
 */

/*********************************************************************
 * CONSTANTS
 */

#define SCALE_SERV_UUID									0x1825
#define SCALE_MEAS_UUID									0x2A95
#define SCALE_SENSOR_LOC_UUID						0x2A97
#define SCALE_CTRL_PT_UUID							0x2A99

// Heart Rate Service Parameters
#define SCALE_MEAS                      0
#define SCALE_MEAS_CHAR_CFG             1
#define SCALE_SENS_LOC                  2
#define SCALE_COMMAND                   3

// Maximum length of heart rate measurement characteristic
#define SCALE_MEAS_MAX                  (ATT_MTU_SIZE -5)

// Values for flags
#define SCALE_FLAGS_FORMAT_UINT16       0x01
#define SCALE_FLAGS_CONTACT_NOT_SUP     0x00
#define SCALE_FLAGS_CONTACT_NOT_DET     0x04
#define SCALE_FLAGS_CONTACT_DET         0x06
#define SCALE_FLAGS_ENERGY_EXP          0x08
#define SCALE_FLAGS_RR                  0x10

// Values for sensor location
#define SCALE_SENS_LOC_OTHER            0x00
#define SCALE_SENS_LOC_CHEST            0x01
#define SCALE_SENS_LOC_WRIST            0x02
#define SCALE_SENS_LOC_FINGER           0x03
#define SCALE_SENS_LOC_HAND             0x04
#define SCALE_SENS_LOC_EARLOBE          0x05
#define SCALE_SENS_LOC_FOOT             0x06

// Value for command characteristic
#define SCALE_COMMAND_ENERGY_EXP        0x01

// ATT Error code
// Control point value not supported
#define SCALE_ERR_NOT_SUP               0x80

// Heart Rate Service bit fields
#define SCALE_SERVICE                   0x00000001

// Callback events
#define SCALE_MEAS_NOTI_ENABLED         1
#define SCALE_MEAS_NOTI_DISABLED        2
#define SCALE_COMMAND_SET               3

/*********************************************************************
 * TYPEDEFS
 */

// Heart Rate Service callback function
typedef void (*scaleServiceCB_t)(uint8 event);

/*********************************************************************
 * MACROS
 */

/*********************************************************************
 * Profile Callbacks
 */


/*********************************************************************
 * API FUNCTIONS 
 */

/*
 * Scale_AddService- Initializes the Heart Rate service by registering
 *          GATT attributes with the GATT server.
 *
 * @param   services - services to add. This is a bit map and can
 *                     contain more than one service.
 */

extern bStatus_t Scale_AddService( uint32 services );

/*
 * Scale_Register - Register a callback function with the
 *          Heart Rate Service
 *
 * @param   pfnServiceCB - Callback function.
 */

extern void Scale_Register( scaleServiceCB_t pfnServiceCB );

/*
 * Scale_SetParameter - Set a Heart Rate parameter.
 *
 *    param - Profile parameter ID
 *    len - length of data to right
 *    value - pointer to data to write.  This is dependent on
 *          the parameter ID and WILL be cast to the appropriate 
 *          data type (example: data type of uint16 will be cast to 
 *          uint16 pointer).
 */
extern bStatus_t Scale_SetParameter( uint8 param, uint8 len, void *value );
  
/*
 * Scale_GetParameter - Get a Heart Rate parameter.
 *
 *    param - Profile parameter ID
 *    value - pointer to data to write.  This is dependent on
 *          the parameter ID and WILL be cast to the appropriate 
 *          data type (example: data type of uint16 will be cast to 
 *          uint16 pointer).
 */
extern bStatus_t Scale_GetParameter( uint8 param, void *value );

/*********************************************************************
 * @fn          Scale_MeasNotify
 *
 * @brief       Send a notification containing a heart rate
 *              measurement.
 *
 * @param       connHandle - connection handle
 * @param       pNoti - pointer to notification structure
 *
 * @return      Success or Failure
 */
extern bStatus_t Scale_MeasNotify( uint16 connHandle, attHandleValueNoti_t *pNoti );

/*********************************************************************
 * @fn          Scale_HandleConnStatusCB
 *
 * @brief       Heart Rate Service link status change handler function.
 *
 * @param       connHandle - connection handle
 * @param       changeType - type of change
 *
 * @return      none
 */
extern void Scale_HandleConnStatusCB( uint16 connHandle, uint8 changeType );

/*********************************************************************
*********************************************************************/

#ifdef __cplusplus
}
#endif

#endif /* SCALESERVICE_H */
