/********************************** (C) COPYRIGHT *******************************
* File Name          : heartrateservice.C
* Author             : WCH
* Version            : V1.0
* Date               : 2018/12/11
* Description        : ʷ
            
*******************************************************************************/

/*********************************************************************
 * INCLUDES
 */
#include "CONFIG.h"
//#include "battservice.h"

#include "scale_service.h"

/*********************************************************************
 * MACROS
 */

/*********************************************************************
 * CONSTANTS
 */

// Position of heart rate measurement value in attribute array
#define SCALE_MEAS_VALUE_POS            2

/*********************************************************************
 * TYPEDEFS
 */

/*********************************************************************
 * GLOBAL VARIABLES
 */
// Heart rate service
CONST uint8 scaleServUUID[ATT_BT_UUID_SIZE] =
{ 
  LO_UINT16(SCALE_SERV_UUID), HI_UINT16(SCALE_SERV_UUID)
};

// Heart rate measurement characteristic
CONST uint8 scaleMeasUUID[ATT_BT_UUID_SIZE] =
{ 
  LO_UINT16(SCALE_MEAS_UUID), HI_UINT16(SCALE_MEAS_UUID)
};

// Sensor location characteristic
CONST uint8 scaleSensLocUUID[ATT_BT_UUID_SIZE] =
{ 
  LO_UINT16(SCALE_SENSOR_LOC_UUID), HI_UINT16(SCALE_SENSOR_LOC_UUID)
};

// Command characteristic
CONST uint8 scaleCommandUUID[ATT_BT_UUID_SIZE] =
{ 
  LO_UINT16(SCALE_CTRL_PT_UUID), HI_UINT16(SCALE_CTRL_PT_UUID)
};

/*********************************************************************
 * EXTERNAL VARIABLES
 */

/*********************************************************************
 * EXTERNAL FUNCTIONS
 */

/*********************************************************************
 * LOCAL VARIABLES
 */

static scaleServiceCB_t scaleServiceCB;

/*********************************************************************
 * Profile Attributes - variables
 */

// Heart Rate Service attribute
static CONST gattAttrType_t scaleService = { ATT_BT_UUID_SIZE, scaleServUUID };

// Heart Rate Measurement Characteristic
// Note characteristic value is not stored here
static uint8 scaleMeasProps = GATT_PROP_NOTIFY;
static uint8 scaleMeas = 0;
static gattCharCfg_t scaleMeasClientCharCfg[GATT_MAX_NUM_CONN];

// Sensor Location Characteristic
static uint8 scaleSensLocProps = GATT_PROP_READ;
static uint8 scaleSensLoc = 0;

// Command Characteristic
static uint8 scaleCommandProps = GATT_PROP_WRITE;
static uint8 scaleCommand = 0;

/*********************************************************************
 * Profile Attributes - Table
 */

static gattAttribute_t scaleAttrTbl[] = 
{
  // Heart Rate Service
  { 
    { ATT_BT_UUID_SIZE, primaryServiceUUID }, /* type */
    GATT_PERMIT_READ,                         /* permissions */
    0,                                        /* handle */
    (uint8 *)&scaleService                /* pValue */
  },

    // Heart Rate Measurement Declaration
    { 
      { ATT_BT_UUID_SIZE, characterUUID },
      GATT_PERMIT_READ, 
      0,
      &scaleMeasProps 
    },

      // Heart Rate Measurement Value
      { 
        { ATT_BT_UUID_SIZE, scaleMeasUUID },
        0, 
        0, 
        &scaleMeas 
      },

      // Heart Rate Measurement Client Characteristic Configuration
      { 
        { ATT_BT_UUID_SIZE, clientCharCfgUUID },
        GATT_PERMIT_READ | GATT_PERMIT_WRITE, 
        0, 
        (uint8 *) &scaleMeasClientCharCfg 
      },      

    // Sensor Location Declaration
    { 
      { ATT_BT_UUID_SIZE, characterUUID },
      GATT_PERMIT_READ, 
      0,
      &scaleSensLocProps 
    },

      // Sensor Location Value
      { 
        { ATT_BT_UUID_SIZE, scaleSensLocUUID },
        GATT_PERMIT_READ, 
        0, 
        &scaleSensLoc 
      },

    // Command Declaration
    { 
      { ATT_BT_UUID_SIZE, characterUUID },
      GATT_PERMIT_READ, 
      0,
      &scaleCommandProps 
    },

      // Command Value
      { 
        { ATT_BT_UUID_SIZE, scaleCommandUUID },
        GATT_PERMIT_WRITE, 
        0, 
        &scaleCommand 
      }
};


/*********************************************************************
 * LOCAL FUNCTIONS
 */
static uint8 scale_ReadAttrCB( uint16 connHandle, gattAttribute_t *pAttr,
                                          uint8 *pValue, uint16 *pLen, uint16 offset,
                                          uint16 maxLen, uint8 method );
static bStatus_t scale_WriteAttrCB( uint16 connHandle, gattAttribute_t *pAttr,
                                           uint8 *pValue, uint16 len, uint16 offset,
                                           uint8 method );

/*********************************************************************
 * PROFILE CALLBACKS
 */
// Heart Rate Service Callbacks
gattServiceCBs_t scaleCBs =
{
  scale_ReadAttrCB,  // Read callback function pointer
  scale_WriteAttrCB, // Write callback function pointer
  NULL                   // Authorization callback function pointer
};

/*********************************************************************
 * PUBLIC FUNCTIONS
 */

/*********************************************************************
 * @fn      Scale_AddService
 *
 * @brief   Initializes the Heart Rate service by registering
 *          GATT attributes with the GATT server.
 *
 * @param   services - services to add. This is a bit map and can
 *                     contain more than one service.
 *
 * @return  Success or Failure
 */
bStatus_t Scale_AddService( uint32 services )
{
  uint8 status = SUCCESS;

  // Initialize Client Characteristic Configuration attributes
  GATTServApp_InitCharCfg( INVALID_CONNHANDLE, scaleMeasClientCharCfg );

  if ( services & SCALE_SERVICE )
  {
    // Register GATT attribute list and CBs with GATT Server App
    status = GATTServApp_RegisterService( scaleAttrTbl, 
                                          GATT_NUM_ATTRS( scaleAttrTbl ),
																					GATT_MAX_ENCRYPT_KEY_SIZE,
                                          &scaleCBs );
  }

  return ( status );
}

/*********************************************************************
 * @fn      Scale_Register
 *
 * @brief   Register a callback function with the Heart Rate Service.
 *
 * @param   pfnServiceCB - Callback function.
 *
 * @return  None.
 */
extern void Scale_Register( scaleServiceCB_t pfnServiceCB )
{
  scaleServiceCB = pfnServiceCB;
}

/*********************************************************************
 * @fn      Scale_SetParameter
 *
 * @brief   Set a Heart Rate parameter.
 *
 * @param   param - Profile parameter ID
 * @param   len - length of data to right
 * @param   value - pointer to data to write.  This is dependent on
 *          the parameter ID and WILL be cast to the appropriate 
 *          data type (example: data type of uint16 will be cast to 
 *          uint16 pointer).
 *
 * @return  bStatus_t
 */
bStatus_t Scale_SetParameter( uint8 param, uint8 len, void *value )
{
  bStatus_t ret = SUCCESS;
  switch ( param )
  {
     case SCALE_MEAS_CHAR_CFG:
      // Need connection handle
      //scaleMeasClientCharCfg.value = *((uint16*)value);
      break;      

    case SCALE_SENS_LOC:
      scaleSensLoc = *((uint8*)value);
      break;

    default:
      ret = INVALIDPARAMETER;
      break;
  }
  
  return ( ret );
}

/*********************************************************************
 * @fn      Scale_GetParameter
 *
 * @brief   Get a Heart Rate parameter.
 *
 * @param   param - Profile parameter ID
 * @param   value - pointer to data to get.  This is dependent on
 *          the parameter ID and WILL be cast to the appropriate 
 *          data type (example: data type of uint16 will be cast to 
 *          uint16 pointer).
 *
 * @return  bStatus_t
 */
bStatus_t Scale_GetParameter( uint8 param, void *value )
{
  bStatus_t ret = SUCCESS;
  switch ( param )
  {
    case SCALE_MEAS_CHAR_CFG:
      // Need connection handle
      //*((uint16*)value) = scaleMeasClientCharCfg.value;
      break;      

    case SCALE_SENS_LOC:
      *((uint8*)value) = scaleSensLoc;
      break;
      
    case SCALE_COMMAND:
      *((uint8*)value) = scaleCommand;
      break;  

    default:
      ret = INVALIDPARAMETER;
      break;
  }
  
  return ( ret );
}

/*********************************************************************
 * @fn          Scale_MeasNotify
 *
 * @brief       Send a notification containing a heart rate
 *              measurement.
 *
 * @param       connHandle - connection handle
 * @param       pNoti - pointer to notification structure
 *
 * @return      Success or Failure
 */
bStatus_t Scale_MeasNotify( uint16 connHandle, attHandleValueNoti_t *pNoti )
{
  uint16 value = GATTServApp_ReadCharCfg( connHandle, scaleMeasClientCharCfg );

  // If notifications enabled
  if ( value & GATT_CLIENT_CFG_NOTIFY )
  {
    // Set the handle
    pNoti->handle = scaleAttrTbl[SCALE_MEAS_VALUE_POS].handle;
  
    // Send the notification
    return GATT_Notification( connHandle, pNoti, FALSE );
  }

  return bleIncorrectMode;
}
                               
/*********************************************************************
 * @fn          scale_ReadAttrCB
 *
 * @brief       Read an attribute.
 *
 * @param       connHandle - connection message was received on
 * @param       pAttr - pointer to attribute
 * @param       pValue - pointer to data to be read
 * @param       pLen - length of data to be read
 * @param       offset - offset of the first octet to be read
 * @param       maxLen - maximum length of data to be read
 *
 * @return      Success or Failure
 */
static uint8 scale_ReadAttrCB( uint16 connHandle, gattAttribute_t *pAttr,
                                          uint8 *pValue, uint16 *pLen, uint16 offset,
                                          uint16 maxLen, uint8 method )
{
  bStatus_t status = SUCCESS;

  uint16 uuid = BUILD_UINT16( pAttr->type.uuid[0], pAttr->type.uuid[1]);
	
  // Make sure it's not a blob operation (no attributes in the profile are long)
  if ( offset > 0 )
  {
    return ( ATT_ERR_ATTR_NOT_LONG );
  }
 

  if (uuid == SCALE_SENSOR_LOC_UUID)
  {
    *pLen = 1;
    pValue[0] = *pAttr->pValue;
  }
  else
  {
    status = ATT_ERR_ATTR_NOT_FOUND;
  }

  return ( status );
}

/*********************************************************************
 * @fn      scale_WriteAttrCB
 *
 * @brief   Validate attribute data prior to a write operation
 *
 * @param   connHandle - connection message was received on
 * @param   pAttr - pointer to attribute
 * @param   pValue - pointer to data to be written
 * @param   len - length of data
 * @param   offset - offset of the first octet to be written
 *
 * @return  Success or Failure
 */
static bStatus_t scale_WriteAttrCB( uint16 connHandle, gattAttribute_t *pAttr,
                                           uint8 *pValue, uint16 len, uint16 offset,
                                           uint8 method )
{
  bStatus_t status = SUCCESS;
 
  uint16 uuid = BUILD_UINT16( pAttr->type.uuid[0], pAttr->type.uuid[1]);
  switch ( uuid )
  {
    case SCALE_CTRL_PT_UUID:
      if ( offset > 0 )
      {
        status = ATT_ERR_ATTR_NOT_LONG;
      }
      else if (len != 1)
      {
        status = ATT_ERR_INVALID_VALUE_SIZE;
      }
      else if (*pValue != SCALE_COMMAND_ENERGY_EXP)
      {
        status = SCALE_ERR_NOT_SUP;
      }
      else
      {
        *(pAttr->pValue) = pValue[0];
        
        (*scaleServiceCB)(SCALE_COMMAND_SET);
        
      }
      break;

    case GATT_CLIENT_CHAR_CFG_UUID:
      status = GATTServApp_ProcessCCCWriteReq( connHandle, pAttr, pValue, len,
                                               offset, GATT_CLIENT_CFG_NOTIFY );
      if ( status == SUCCESS )
      {
        uint16 charCfg = BUILD_UINT16( pValue[0], pValue[1] );

        (*scaleServiceCB)( (charCfg == GATT_CFG_NO_OPERATION) ?
                                SCALE_MEAS_NOTI_DISABLED :
                                SCALE_MEAS_NOTI_ENABLED );
      }
      break;       
 
    default:
      status = ATT_ERR_ATTR_NOT_FOUND;
      break;
  }

  return ( status );
}

/*********************************************************************
 * @fn          Scale_HandleConnStatusCB
 *
 * @brief       Heart Rate Service link status change handler function.
 *
 * @param       connHandle - connection handle
 * @param       changeType - type of change
 *
 * @return      none
 */
void Scale_HandleConnStatusCB( uint16 connHandle, uint8 changeType )
{ 
  // Make sure this is not loopback connection
  if ( connHandle != LOOPBACK_CONNHANDLE )
  {
    // Reset Client Char Config if connection has dropped
    if ( ( changeType == LINKDB_STATUS_UPDATE_REMOVED )      ||
         ( ( changeType == LINKDB_STATUS_UPDATE_STATEFLAGS ) && 
           ( !linkDB_Up( connHandle ) ) ) )
    { 
      GATTServApp_InitCharCfg( connHandle, scaleMeasClientCharCfg );
    }
  }
}


/*********************************************************************
*********************************************************************/
